import { z } from "zod";

export type UnknownRecord = Record<string, unknown>;

export interface CLIFlags {
  help: boolean;
  verbose: boolean;
  stdout: boolean;
}

export interface CLIArguments {
  inputFilePath: string;
  flags: CLIFlags;
}

export const BindingSchema = z.object({
  contentKey: z.string().default("content"),
  sources: z
    .array(z.string())
    .default(["**/*.yaml", "**/*.md", "**/*.toml", "**/*.json"]),
  options: z.record(z.string(), z.any()).default({}),
  processors: z.array(z.string()).default([]),
  renderer: z.string().optional(),
});

export type MuseProcessorFn = (
  binding: Binding,
  opts: Record<string, unknown>,
  flags: CLIFlags,
) => Promise<Binding>;

export type MuseRenderFn = (
  binding: Binding,
  opts: UnknownRecord,
  flags: CLIFlags,
) => Promise<boolean>;

export interface MuseProcessor {
  readonly filePath: string;
  readonly name: string;
  readonly description: string;
  readonly process: MuseProcessorFn;
}

export interface MuseEntry<MetaShape = UnknownRecord> {
  _raw: string;
  filePath: string;
  data: Record<string, unknown>;
  meta: MetaShape;
}

export interface Binding<MetaShape = UnknownRecord> {
  readonly _raw: z.infer<typeof BindingSchema>;
  readonly bindingPath: string;
  readonly contentKey: string;
  readonly entries: MuseEntry[];
  readonly processors: MuseProcessor[];
  readonly meta: MetaShape;
  readonly options: UnknownRecord;
}
